// Written by Hangzai Luo
// This file is distributed under GPL v 2.0 or higher.

%option C++ noyywrap 8bit batch caseless never-interactive prefix="eng"

%{
#include <string.h>
#include <ctype.h>
#include "engLexerType.h"
#define RETURN_SENT return 1
%}

ws			[ \t\a\b\f\v\r\n]+
trueSpace	[ \t]
dig			[0-9]
special		[^ \t\a\b\f\v\ra-zA-Z0-9]
nonSig		[^a-zA-Z0-9]
alpha		[A-Za-z]
Apos	    (\')
AposExt 	(\'|"&apos;")
sentEnd		([\!\-\;\,\.\?]|"--"|"...")
hexChar     [0-9a-fA-F]

utf8Music   \xe2\x99\xaa
utf8Musics  {utf8Music}+

timeTail    ("a.m."|"am"|"p.m."|"pm")
yearTail    ("a.c."|"ac"|"b.c."|"bc")
numYear     {Apos}{dig}{2}

timeFmt     ({dig}{1,2})((\:{dig}{1,2}){1,2})(([ ]*){timeTail})?
yearFmt     ({dig}+|{numYear})[s]?([ ]*{yearTail})?

syncStr     {hexChar}{16}\-{hexChar}{16}\:([\-NSR])\:{hexChar}{8}\:

lpNEHead	"<ENAMEX"
lpNETail	"</ENAMEX>"
xmlTagChar	[\<\>]
xmlComChar	[^\<\>]
xmlComCharE	{xmlComChar}|[\r\n]
xmlQoute	"&quot;"
xmlAnd		"&amp;"
xmlApos		"&apos;"
xmlGt		"&gt;"
xmlLt		"&lt;"

xmlPOS		"&apos;s"
xmlHad		"&apos;d"

Im          ("I"{AposExt}"m")

lpNETag		{lpNEHead}{xmlComChar}+\>{xmlComCharE}+{lpNETail}
lpSyncTag	"<SYNC>"{xmlComChar}+"</SYNC>"
lpSentTag	"<sent>"
lpSentEnd	"</sent>"
lpDocStart	"<?xml"{xmlComChar}+\>{xmlComChar}*"<DOCUMENT>"
lpDocEnd	"</DOCUMENT>"

lpSyncTag2	"<SYNC id="{xmlComChar}+">"

lpSentTag2	"<s"{xmlComChar}+">"
lpSentEnd2	"</s>"
lpDocStart2	"<?xml"{xmlComChar}+\>{xmlComChar}*"<output>"
lpDocEnd2	"</output>"

lpNewsSep	"<news"{xmlComChar}+\>
lpNewsEnd	"</news>"

nonAscii	[^\x00-\x7f]

numWords	("hundred"|"thousand"|"million"|"billion"|"trillion")

numConnect  ([ ]+)|\-

numOneComm	("six"|"seven"|"nine")
numOneIrr	("one"|"two"|"three"|"four"|"five"|"eight")
numOneRoot	("thir"|"fif"|"eigh")
numTeenRoot ({numOneRoot}|{numOneComm}|"four")
numTyRoot   ({numOneRoot}|{numOneComm}|"for")
numTwoIrr	("ten"|"eleven"|"twelve"|"twenty")
numTwoComp	{numTeenRoot}("teen")
numTens		{numTyRoot}("ty")
numSmallC	({numTens}|"twenty"){numConnect}({numOneComm}|{numOneIrr})

numSmall	({numSmallC}|{numTens}|{numTwoComp}|{numTwoIrr}|{numOneComm}|{numOneIrr})

numELarge1	{numSmall}({numConnect}{numWords})+
numELarge2	{numELarge1}({numConnect}{numELarge1})+({numConnect}("and"{numConnect})?{numSmall})?

numInt		{dig}+((\,({dig}{2,4}))*)
numFlt  	{numInt}*\.{dig}+
numTel		([+-]?){dig}+([-]{dig}+)*

numDollar	\$({alpha}{1,2})?({numInt}|{numFlt})
numPercent	({numFlt}|{dig}+)([ ]*)("%"|"percent")

numELarge3	({numFlt}|{numInt}|{numDollar}){numConnect}({numELarge1}|{numELarge2}|{numWords})

numHead		("several"|"multiple"|"few"|"a few"|"quite a few"|"a")
numTimes	({numInt}|{numFlt})[ ]+("times")

numEng		({numELarge1}|{numELarge2}|{numELarge3}|({numHead}({numConnect}{numWords})+)|{numWords}|{numTimes}|{numSmall})

seqTail		("st"|"nd"|"rd"|"th")
seqNum		{dig}+{seqTail}

seqRoot	    ("four"|"fif"|"six"|"seven"|"eigh"|"nin")
seqOne	    {seqRoot}("th")
seqOneSpec	("first"|"second"|"third")
seqTwoRoot	("ten"|"eleven"|"twelf"|{numTwoComp})
seqTwo		({seqTwoRoot}|({numTyRoot}"tie")|"twentie")"th"
seqSmallC	({numTens}|"twenty")(([ ]+)|\-)({seqOne}|{seqOneSpec})

seqSmall	({seqSmallC}|{seqTwo}|{seqOne}|{seqOneSpec}|{seqNum})

month3		("jan"|"feb"|"mar"|"apr"|"jun"|"jul"|"aug"|"sep"|"oct"|"nov"|"dec")
month3d		({month3}[\.])
monthFull	("january"|"febuary"|"march"|"april"|"may"|"june"|"july"|"august"|"september"|"october"|"november"|"december")
month		({month3}|{month3d}|{monthFull})

weekShort   ("mon"|"tue"|"tues"|"wed"|"thu"|"thur"|"fri"|"sat"|"sun")
weekShortD  {weekShort}[\.]
weekFull    ("monday"|"tuesday"|"wednesday"|"thursday"|"friday"|"saturday"|"sunday")
weekAll     {weekFull}|{weekShortD}|{weekShort}

numDay1		{numInt}\/{numInt}((\/{numInt})?)
numDay2		{numInt}\.{numInt}\.{numInt}
numDay3		{numInt}\/{numInt}\,trueSpace*{numInt}
numDay		{numDay1}|{numDay2}|{numDay3}

monthDay1	({month}[ ]+({seqSmall}|{dig}+))
monthDay2	{seqSmall}[ ]+"of"[ ]+{month}
monthDay	({monthDay1}|{monthDay2})
monthYear1	{month}"\,"[ ]*{yearFmt}
monthYear2	{month}[ ]+("of"[ ]+){yearFmt}
monthYear	{monthYear1}|{monthYear2}

monthDayYear	{monthDay}([\,]?[ ]*){yearFmt}

engDateGen	{monthYear}|{monthDayYear}|{monthDay}|{month}|{yearFmt}|{timeFmt}
engDateExt  ({weekAll}[\,]?[ ]+)?{engDateGen}
engDate     {engDateExt}|{weekFull}

numAll		{numPercent}|{numDollar}|{numInt}|{numFlt}|{numTel}

midInitial	{alpha}\.
acronym		{midInitial}{2,}

engChar  	({alpha}|{nonAscii})
engCharNo  	([a-ce-rt-z]|{nonAscii})

engWord1	{engChar}({engChar}|{dig})*(\-({engChar}|{dig})+)*
extName1	{engChar}+{Apos}(({engCharNo}{engChar}*)|([ds]{engChar}+))

numWord 	{dig}+([\-]?){engWord1}

engWord		{numWord}|{engWord1}|{extName1}

engShort	("Adm."|"Ala."|"Ariz."|"Ark."|"Aug."|"Ave."|"Bancorp."|"Bhd."|"Brig."|"Bros."|"CO."|"CORP."|"COS."|"Ca."|"Calif."|"Canada-U.S."|"Canadian-U.S."|"Capt."|"Cia."|"Cie."|"Co."|"Col."|"Colo."|"Conn."|"Corp."|"Cos."|"D-Mass."|"Dec."|"Del."|"Dept."|"Dr."|"Drs."|"Etc."|"Feb."|"Fla."|"Ft."|"Ga."|"Gen."|"Gov."|"Hon."|"INC."|"Ill."|"Inc."|"Ind."|"Jan."|"Japan-U.S."|"Jr."|"Kan."|"Korean-U.S."|"Ky."|"La."|"Lt."|"Ltd."|"Maj."|"Mass."|"Md."|"Messrs."|"Mfg."|"Mich."|"Minn."|"Miss."|"Mo."|"Mr."|"Mrs."|"Ms."|"Neb."|"Nev."|"No."|"Nos."|"Nov."|"Oct."|"Okla."|"Ont."|"Ore."|"Pa."|"Ph."|"Prof."|"Prop."|"Pty."|"Rep."|"Reps."|"Rev."|"Sen."|"Sens."|"Sept."|"Sgt."|"Sino-U.S."|"Sr."|"St."|"Ste."|"Tenn."|"Tex."|"U.S.-U.K."|"U.S.-U.S.S.R."|"Va."|"Vt."|"W.Va."|"Wash."|"Wis."|"Wyo."|"anti-U.S."|"cap."|"days."|"etc."|"ft."|"non-U.S."|"dept."|"president-U.S."|"vs.")

engPhrase1	(("abc"([ ]+)"news")|("african"([ ]+)"union")|("air"([ ]+)"force")|("al"([ ]+)"anbar")|("al"([ ]+)"qaeda")|("american"([ ]+)"airlines")|("atomic"([ ]+)"energy")|("avian"([ ]+)"flu")|("avian"([ ]+)"influenza")|("baptist"([ ]+)"church")|("bay"([ ]+)"area")|("bin"([ ]+)"laden")|("bird"([ ]+)"flu")|("capitol"([ ]+)"hill")|("car"([ ]+)"bomb")|("carbon"([ ]+)"dioxide")|("catholic"([ ]+)"church")|("cbs"([ ]+)"news")|("cell"([ ]+)"phone")|("central"([ ]+)"government")|("civil"([ ]+)"war")|("coast"([ ]+)"guard")|("county"([ ]+)"sheriff")|("credit"([ ]+)"card")|("credit"([ ]+)"cards")|("crude"([ ]+)"oil")|("czech"([ ]+)"republic")|("da"([ ]+)"vinci")|("de"([ ]+)"la")|("democratic"([ ]+)"party")|("deutsche"([ ]+)"boerse")|("dow"([ ]+)"jones")|("dwyane"([ ]+)"wade")|("east"([ ]+)"london")|("east"([ ]+)"timor")|("european"([ ]+)"union")|("fatah"([ ]+)"movement"))
engPhrase2	(("ford"([ ]+)"motor")|("fox"([ ]+)"news")|("gaza"([ ]+)"city")|("gaza"([ ]+)"strip")|("general"([ ]+)"motors")|("goldman"([ ]+)"sachs")|("guantanamo"([ ]+)"bay")|("gulf"([ ]+)"coast")|("hard"([ ]+)"drive")|("high"([ ]+)"school")|("homeland"([ ]+)"security")|("hong"([ ]+)"kong")|("identity"([ ]+)"theft")|("intellectual"([ ]+)"property")|("islamic"([ ]+)"courts")|("islamic"([ ]+)"republic")|("jewish"([ ]+)"state")|("kansas"([ ]+)"city")|("la"([ ]+)"fuente")|("las"([ ]+)"vegas")|("latin"([ ]+)"america")|("latin"([ ]+)"american")|("long"([ ]+)"island")|("los"([ ]+)"angeles")|("memorial"([ ]+)"day")|("mexico"([ ]+)"city")|("middle"([ ]+)"east")|("morgan"([ ]+)"stanley")|("mount"([ ]+)"merapi")|("natural"([ ]+)"gas")|("new"([ ]+)"york")|("north"([ ]+)"america")|("north"([ ]+)"american")|("north"([ ]+)"carolina")|("north"([ ]+)"korea"))
engPhrase3	(("north"([ ]+)"korean")|("northern"([ ]+)"ireland")|("oklahoma"([ ]+)"city")|("puerto"([ ]+)"rico")|("real"([ ]+)"estate")|("red"([ ]+)"cross")|("red"([ ]+)"sox")|("road"([ ]+)"map")|("san"([ ]+)"antonio")|("san"([ ]+)"diego")|("san"([ ]+)"francisco")|("san"([ ]+)"jose")|("santa"([ ]+)"clara")|("saudi"([ ]+)"arabia")|("security"([ ]+)"council")|("south"([ ]+)"africa")|("south"([ ]+)"african")|("south"([ ]+)"america")|("south"([ ]+)"american")|("south"([ ]+)"carolina")|("south"([ ]+)"dakota")|("south"([ ]+)"korea")|("south"([ ]+)"korean")|("southern"([ ]+)"afghanistan")|("southern"([ ]+)"california")|("soviet"([ ]+)"union")|("space"([ ]+)"shuttle")|("space"([ ]+)"station")|("sri"([ ]+)"lanka")|("sri"([ ]+)"lankan")|("st."([ ]+)"louis")|("st."([ ]+)"petersburg")|("stock"([ ]+)"exchange")|("sunni"([ ]+)"arab")|("sunni"([ ]+)"arabs")|("super"([ ]+)"bowl")|("tamil"([ ]+)"tiger")|("tamil"([ ]+)"tigers")|("time"([ ]+)"warner")|("united"([ ]+)"kingdom"))
engPhrase4	(("united"([ ]+)"nations")|("united"([ ]+)"states")|("usa"([ ]+)"today")|("van"([ ]+)"der")|("van"([ ]+)"dyk")|("vice"([ ]+)"president")|("wall"([ ]+)"street")|("washington"([ ]+)"post")|("web"([ ]+)"site")|("web"([ ]+)"sites")|("west"([ ]+)"bank")|("west"([ ]+)"virginia")|("white"([ ]+)"house")|("white"([ ]+)"sox")|("winged"([ ]+)"foot")|("world"([ ]+)"bank")|("world"([ ]+)"war")|("yankee"([ ]+)"stadium")|("zip"([ ]+)"code"))

engPhraseAddr   ("gaza"([ ]+)"strip")|("goldman"([ ]+)"sachs")|("guantanamo"([ ]+)"bay")|("gulf"([ ]+)"coast")|("hong"([ ]+)"kong")|("islamic"([ ]+)"republic")|("jewish"([ ]+)"state")|("kansas"([ ]+)"city")|("la"([ ]+)"fuente")|("las"([ ]+)"vegas")|("long"([ ]+)"island")|("los"([ ]+)"angeles")|("mexico"([ ]+)"city")|("middle"([ ]+)"east")|("morgan"([ ]+)"stanley")|("new"([ ]+)"york")|("oklahoma"([ ]+)"city")|("puerto"([ ]+)"rico")|("real"([ ]+)"estate")|("san"([ ]+)"antonio")|("san"([ ]+)"diego")|("san"([ ]+)"francisco")|("san"([ ]+)"jose")|("santa"([ ]+)"clara")|("saudi"([ ]+)"arabia")|("soviet"([ ]+)"union")|("sri"([ ]+)"lanka")|("sri"([ ]+)"lankan")|("st."([ ]+)"louis")|("st."([ ]+)"petersburg")|("sunni"([ ]+)"arab")|("sunni"([ ]+)"arabs")|("tamil"([ ]+)"tiger")|("tamil"([ ]+)"tigers")|("united"([ ]+)"kingdom")|("united"([ ]+)"nations")|("united"([ ]+)"states")|("van"([ ]+)"der")|("van"([ ]+)"dyk")|("wall"([ ]+)"street")|("west"([ ]+)"bank")|("white"([ ]+)"house")

engPhrase	{engPhrase1}|{engPhrase2}|{engPhrase3}|{engPhrase4}

engWordEx	{engWord}|{engShort}|{acronym}|{engPhraseAddr}

engAposTail	("ll"|"re"|"ve"|"t")

engAPDef	({engWordEx}{AposExt}{engAposTail})
engWQKeep	({Apos}("s"|"d"))

engWQHav	"havn"{AposExt}"t"
engWQWill	"won"{AposExt}"t"
engWQCan	"can"{AposExt}"t"

extName2	{engChar}+{xmlApos}(({engCharNo}{engChar}*)|([ds]{engChar}+))

%%

{ws}	                    /* skip blanks and tabs */

{syncStr}                   HandleStr(YYText(),ewCCSyncStr); RETURN_SENT;

{utf8Musics}                HandleStr(YYText(),ewPuncMusic); RETURN_SENT;

{xmlQoute}                  HandleStr("\"",ewPunc); RETURN_SENT;

{xmlAnd}                    HandleStr("&",ewPunc); RETURN_SENT;

{xmlApos}                   HandleStr("'",ewPunc); RETURN_SENT;

{xmlGt}                     HandleStr("<",ewPunc); RETURN_SENT;

{xmlLt}                     HandleStr(">",ewPunc); RETURN_SENT;

{xmlPOS}/{nonSig}           HandleStr("'s",ewWordQoute); RETURN_SENT;

{xmlHad}/{nonSig}           HandleStr("'d",ewWordQoute); RETURN_SENT;

{Im}/{nonSig}               HandleStr("I",ewWord);HandleStr("am",ewWord); RETURN_SENT;

{lpNETag}					{
	int len = YYLeng();
	for(int i=0;i<len;i++)
	{
		if( (yytext[i]=='\r') || (yytext[i]=='\n') )
			yytext[i] = ' ';
	}
	HandleStr(YYText(),ewLPTagNE); RETURN_SENT;
}

{lpSyncTag}|{lpSyncTag2}    HandleStr(YYText(),ewLPTagSync); RETURN_SENT;

{lpNewsSep}					HandleStr(YYText(),ewLPTagNews); RETURN_SENT;

{lpNewsEnd}					HandleStr(YYText(),ewLPTagNewsEnd); RETURN_SENT;

{lpSentTag}|{lpSentTag2}	HandleStr(YYText(),ewLPTagSent); RETURN_SENT;

{lpSentEnd}|{lpSentEnd2}	HandleStr(YYText(),ewLPTagSentEnd); RETURN_SENT;

{lpDocStart}|{lpDocStart2}	HandleStr(YYText(),ewLPTagDocStart); RETURN_SENT;

{lpDocEnd}|{lpDocEnd2}		HandleStr(YYText(),ewLPTagDocEnd); RETURN_SENT;

{engWQHav}/{nonSig}			HandleStr("have",ewWord);HandleStr("not",ewWord); RETURN_SENT;

{engWQWill}/{nonSig}		HandleStr("will",ewWord);HandleStr("not",ewWord); RETURN_SENT;

{engWQCan}/{nonSig}			HandleStr("cannot",ewWord); RETURN_SENT;

{AposExt}"em"/{nonSig}	    HandleStr("them",ewWord); RETURN_SENT;

{numAll}					HandleStr(YYText(),ewNumber); RETURN_SENT;

{engShort}|{engPhraseAddr}	HandleStr(YYText(),ewWord); RETURN_SENT;

{engDate}       			HandleStr(YYText(),ewDate); RETURN_SENT;

{numEng}					HandleStr(YYText(),ewEngNumber); RETURN_SENT;

{seqSmall}					HandleStr(YYText(),ewEngSeqNumber); RETURN_SENT;

{numDay}        			HandleStr(YYText(),ewNumberDate); RETURN_SENT;

{engAPDef}/{nonSig}			{
	int wLen = YYLeng()-3;
	char *type = NULL;

	switch(YYText()[wLen+1])
	{
		case 'l':
		case 'L':
			type = "will";
			break;
		case 'r':
		case 'R':
			type = "are";
			break;
		case 'v':
		case 'V':
			type = "have";
			break;
        default:
            type = "not";
	}

    if(wLen>0)
    {
        if( (yytext[wLen]!='\'')&&(toupper(yytext[wLen])!='N') ) // must be xml entity
            wLen -= 5;

        char oldStore = yytext[wLen];

        yytext[wLen] = 0;
        HandleStr(yytext,ewWord);
        yytext[wLen] = oldStore;
    }
    
	HandleStr(type,ewWord);
	RETURN_SENT;
}

{midInitial}|{acronym}		|
{engWord}					HandleStr(YYText(),ewWord); RETURN_SENT;

{engWQKeep}/{nonSig}		{
    int len = YYLeng();
    if(yytext[len-1]=='S')
        yytext[len-1] = 's';
    else if(yytext[len-1]=='D')
        yytext[len-1] = 'd';
    HandleStr(YYText(),ewWordQoute);
	RETURN_SENT;
}

{extName2}                  {
    const char *txt = YYText();
    const char *pos = strstr(txt,"&apos;");

    if(pos)
    {
        int   len = YYLeng();
        char *str = new char[len+1];

        int parLen = pos-txt;
        memcpy(str,txt,parLen);
        str[parLen] = '\'';

        int parLen2 = len - parLen - 6;

        memcpy(str+parLen+1,pos+6,parLen2);
        str[len-5] = 0;

        HandleStr(str,ewWord);

        delete str;
    }
    else
    {
        HandleStr(txt,ewWord);
    }
	RETURN_SENT;
}

{sentEnd}					HandleStr(YYText(),ewPuncStop); RETURN_SENT;

{special}					HandleStr(YYText(),ewPunc); RETURN_SENT;

%%

